@@ -1,4 +1,4 @@
-// Copyright (c) 2025 Proton AG
+// Copyright (c) 2026 Proton AG
 //
 // This file is part of Proton Mail Bridge.
 //
@@ -42,6 +42,7 @@ import (
 	"github.com/ProtonMail/proton-bridge/v3/internal/events"
 	"github.com/ProtonMail/proton-bridge/v3/internal/focus"
 	"github.com/ProtonMail/proton-bridge/v3/internal/identifier"
+	"github.com/ProtonMail/proton-bridge/v3/internal/platform"
 	"github.com/ProtonMail/proton-bridge/v3/internal/safe"
 	"github.com/ProtonMail/proton-bridge/v3/internal/sentry"
 	"github.com/ProtonMail/proton-bridge/v3/internal/services/imapsmtpserver"
@@ -57,6 +58,7 @@ import (
 	"github.com/bradenaw/juniper/xslices"
 	"github.com/elastic/go-sysinfo/types"
 	"github.com/go-resty/resty/v2"
+	uuid "github.com/google/uuid"
 	"github.com/sirupsen/logrus"
 )
 
@@ -166,6 +168,7 @@ func New(
 	updater Updater, // the updater to fetch and install updates
 	curVersion *semver.Version, // the current version of the bridge
 	keychains *keychain.List, // usable keychains
+	obsService *observability.Service,
 
 	apiURL string, // the URL of the API to use
 	cookieJar http.CookieJar, // the cookie jar to use
@@ -204,6 +207,7 @@ func New(
 		keychains,
 		panicHandler,
 		reporter,
+		obsService,
 
 		api,
 		identifier,
@@ -240,6 +244,7 @@ func newBridge(
 	keychains *keychain.List,
 	panicHandler async.PanicHandler,
 	reporter reporter.Reporter,
+	obsService *observability.Service,
 
 	api *proton.Manager,
 	identifier identifier.Identifier,
@@ -271,9 +276,9 @@ func newBridge(
 		return nil, fmt.Errorf("failed to create focus service: %w", err)
 	}
 
-	unleashService := unleash.NewBridgeService(ctx, api, locator, panicHandler)
+	unleashService := unleash.NewBridgeService(ctx, api, locator, panicHandler, vault.GetFeatureFlagStickyKey())
 
-	observabilityService := observability.NewService(ctx, panicHandler)
+	obsService.Initialize(ctx, panicHandler)
 
 	bridge := &Bridge{
 		vault: vault,
@@ -315,11 +320,11 @@ func newBridge(
 		lastVersion: lastVersion,
 
 		tasks:       tasks,
-		syncService: syncservice.NewService(panicHandler, observabilityService),
+		syncService: syncservice.NewService(panicHandler, obsService),
 
 		unleashService: unleashService,
 
-		observabilityService: observabilityService,
+		observabilityService: obsService,
 
 		notificationStore: notifications.NewStore(locator.ProvideNotificationsCachePath),
 
@@ -334,7 +339,8 @@ func newBridge(
 		reporter,
 		uidValidityGenerator,
 		&bridgeIMAPSMTPTelemetry{b: bridge},
-		observabilityService,
+		obsService,
+		unleashService,
 	)
 
 	// Check whether username has changed and correct (macOS only)
@@ -685,7 +691,7 @@ func (bridge *Bridge) HasAPIConnection() bool {
 // then we verify whether the gluon cache exists using the "new" username (provided by the DB path in this case)
 // if so we modify the cache directory in the user vault.
 func (bridge *Bridge) verifyUsernameChange() {
-	if runtime.GOOS != "darwin" {
+	if runtime.GOOS != platform.MACOS {
 		return
 	}
 
@@ -746,7 +752,7 @@ func (bridge *Bridge) PushObservabilityMetric(metric proton.ObservabilityMetric)
 	bridge.observabilityService.AddMetrics(metric)
 }
 
-func (bridge *Bridge) PushDistinctObservabilityMetrics(errType observability.DistinctionErrorTypeEnum, metrics ...proton.ObservabilityMetric) {
+func (bridge *Bridge) PushDistinctObservabilityMetrics(errType observability.DistinctionMetricTypeEnum, metrics ...proton.ObservabilityMetric) {
 	bridge.observabilityService.AddDistinctMetrics(errType, metrics...)
 }
 
@@ -784,3 +790,7 @@ func (bridge *Bridge) SetHostVersionGetterTest(fn func(host types.Host) string)
 func (bridge *Bridge) SetRolloutPercentageTest(rollout float64) error {
 	return bridge.vault.SetUpdateRollout(rollout)
 }
+
+func (bridge *Bridge) GetFeatureFlagStickyKey() uuid.UUID {
+	return bridge.vault.GetFeatureFlagStickyKey()
+}
